from unittest import main
from test_wisard import WisardTestCase
from test_discriminator import DiscriminatorTestCase
from test_cluswisard import ClusWisardTestCase
from test_kernel_canvas import KernelCanvasTestCase

if __name__ == '__main__':
    main(verbosity=2)
